@@ -819,7 +819,7 @@ dun_start (PluginInfo *info)
 		g_message ("%s: calling Connect...", __func__);
 
 		/* Watch for BT device property changes */
-		dbus_g_object_register_marshaller (nma_marshal_VOID__STRING_BOXED,
+		dbus_g_object_register_marshaller (_nma_marshal_VOID__STRING_BOXED,
 		                                   G_TYPE_NONE,
 		                                   G_TYPE_STRING, G_TYPE_VALUE,
 		                                   G_TYPE_INVALID);
@@ -944,7 +944,12 @@ plugin_info_destroy (gpointer data)
 		gtk_spinner_stop (GTK_SPINNER (info->spinner));
 	g_object_unref (info->settings);
 	g_object_unref (info->btmodel);
+
+	g_signal_handlers_disconnect_matched (info->btclient,
+	                                      G_SIGNAL_MATCH_DATA, 0, 0, NULL,
+	                                      NULL, info);
 	g_object_unref (info->btclient);
+
 	if (info->bus)
 		dbus_g_connection_unref (info->bus);
 	memset (info, 0, sizeof (PluginInfo));
